!
! Module: eexpchipmap_pcms_mod
!
! Authors:  Ian Stewart - SSC
!          
! Description: ***
! 
! Comments: 
! 
! $Id: module.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!- 
!

module eexpchipmap_pcms_mod

! You should write the module name in the 'use' statement, the package name
! (and version number) in the DEPEND file, and the * from the lib*.a name in
! <package>/src/Makefile.
  use cal
  use dal
  use param  
  use errorHandling
  use dss
  use caloaldefs	! for SpacecraftAttitudeType

  use exposure_map_pcms
!  use exposure_aux

  use binned_att
  use cube_aux		! for keyInfoType
  use eimchip2sky_aux	! for rebinChipImagesToSky.

  use cal_aux
  use constants
  use coordinate	! for rawToTel
  use dal_aux		! for readArrayData
!  use dss_aux		! for readGtisFromDss
  use dss_ranges_aux	! for integrateRanges
  use intervals_aux	! for gtiVecType
  use exposure
  use save_image
  use ssc_misc		! for hexStrToIntBitMask
  use wcs_aux
use dump

  implicit none 

  private 

  public eexpchipmap_pcms

  ! Command-line parameters:
  character(256) ::&
!    dssSetName = '',&
    templateImageSetName = '',&
    eventSetName = '',&
!!!    spectrumSetTabName = '',&
    binnedAttSetTabName = '',&
    flatExpMapSetName = '',&
    vigExpMapSetName = '',&
    flatExpMapCubeSetName='',&
    vigExpMapCubeSetName='',&
    tempImageSetName='',&
    tempEventListSetName=''
  character(20) ::&
!    dssStyle='',&
    outputStyle = '',&
    attitudeStyle = ''
  logical ::&
!!!    withSpectrumSet = .false.,&
!    withDss=.false.,&
    doApprox=.false.,&
    withVigExpCubeSet=.false.,&
    withVigExpMapSet=.false.

character(10) ::&
  flagMaskHexStr=''
integer(int32) ::&
  patternLo=0,&
  patternHi=0,&
  flagMask=0,&
  numUserEeVRanges=0,&
  numGtiTabs=0,&
  rawYLo=0
type(RangeT), allocatable ::&
  userEeVRanges(:)
logical ::&
  withNode1=.false.,&
  withGtis=.false.
character(256), allocatable ::&
  userGtiSetTabNames(:,:)

  ! Global variables (these also defined in lccorr_include):
  integer(int16), parameter ::&
    numMicropixelDivs = 5 ! used in call to calcCcdExpMap.
  logical(bool), parameter ::&
    pleaseCorrectDither = .true. ! used in call to calcCcdExpMap.
  integer(int32), parameter ::&
    numESamples = 50, numThetaSamples = 50, numPhiSamples = 36
!  character(256) ::&
!    binnedAttSetName = ''
!  character(20) ::&
!    binnedAttTabName = ''

contains 

  !---------------------------------------------------------------------
  subroutine eexpchipmap_pcms
  !---------------------------------------------------------------------

    implicit none

    type(DataSetT) :: eventSet, set
    type(ArrayT) :: imArray
!, dssSet
!    type(BlockT) :: dssBlock
!    type(DssT) :: dssPointer
    real(single), pointer :: templateImage(:,:)
    integer(int32) :: instrumentId, filtId, requiredCorrections, dummyInstrumentId
    integer :: numCanonicalPatterns, numImages, i, foreignStatus, xiSize, yiSize, numAttBins
    type(localTAWtype) :: timeAttWcs
    integer(int8) ::&
      numCcds, numNodes, ccdNum, nodeNum, ccdNodeNum, ccdnr, maxCcdNodeNum,&
      ccdNodeCombo, numCcdNodeCombos
    integer(int8), allocatable :: ccd2ccdnr(:,:)
!, ccdNumList(:)
    type(keyInfoType), allocatable :: keyInfo(:)
    character(2), allocatable :: ccdStr(:,:)
    type(RangeT) :: EeVRange, calEeVRange
    real(double) :: EeVsamples(numESamples), expStartTimeStamp
    real(single) ::&
      spectrum(numESamples), effArea(numESamples), filterTrans(numESamples),&
      EeVFunctions(numESamples), vigEfunctionAtSrc(numESamples),&
      eNorm, ccdFlatExp
    real(single), allocatable ::&
      ccdExpMaps(:,:,:), qeArray(:,:), tempCcdExpMap(:,:), ccdVigMaps(:,:,:),&
      vigSamplesCube(:,:,:), flatExpMap(:,:), vigExpMap(:,:), binnedAttWeights(:,:)
    integer(int16) ::&
      qePixelXi, qePixelYi, ti, pi,&
      chipMinRawX, chipMaxRawX, chipMinRawY, chipMaxRawY
    type(gtiVecType), allocatable :: gtiVec(:)
    type(expInfoType), allocatable :: expInfo(:,:)
    type(SpacecraftAttitudeType) :: scAttitude, instrumAttitude
    type(TableT) :: tab
    type(WcsT) :: wcs
    real(double), allocatable ::&
      thetaArcsecSamples(:), phiSamples(:),&
      ccdPhi(:,:), ccdThetaArcsec(:,:)
    integer(int16), allocatable ::&
      ccdXRaw(:,:), ccdYRaw(:,:), ccdXRawVec(:), ccdYRawVec(:)
    real(single), allocatable :: vigSamples2d(:,:)
    type(RangeT), pointer :: EeVRanges(:)
    character(256) :: binnedAttSetName = ''
    character(20) :: binnedAttTabName = ''
    type(attBinType), pointer :: binnedScAtt(:)
    type(SpacecraftAttitudeType) ::&
      medianScAttitude, dummyInstrumAttitude
    logical :: noWeightColumn
logical :: testFlag = .false.

    !--------------------------------------------------------------------
    ! Start:

   call readParameters

    call hexStrToIntBitMask(flagMaskHexStr, flagMask, foreignStatus)
    ! in ssclib/ssc_misc.
if (foreignStatus .ne. 0) call error('badHexStrToBitMaskStatus',"")

    if (outputStyle .eq. 'sky') then
      !.................................................................
      ! Read some essentials from the template image:

      call message(AppMsg, verbose, "Reading data from template image.")

      set = dataSet(templateImageSetName, READ)
      imArray = array(set, 0)
      call readWcs(set, imArray, wcs) ! in ssclib/wcs_aux.

      call getCalInfo(set, dummyInstrumentId&
        , expStartTimeStamp, medianScAttitude, dummyInstrumAttitude)
      ! in ssclib/cal_aux.

      call readArrayData(imArray, templateImage)
      ! in ssclib/dal_aux.

      call release(set)

      xiSize = size(templateImage, 1)
      yiSize = size(templateImage, 2)
      deallocate(templateImage)

!*** NOTE! the only things that the template image supplies are (i) wcs, (ii) image pixel dimensions, (iii) median attitude and (iv) exposure start time stamp. It would be nice to have some facility to input the last of these in other ways.

      allocate(flatExpMap(xiSize,yiSize))
      flatExpMap = 0.0
      if (withVigExpMapSet) then
        allocate(vigExpMap(xiSize,yiSize))
        vigExpMap = 0.0
      end if
    end if

    !....................................................................
    ! DSS and cal setups:

    call message(AppMsg, Verbose, "Opening the event list.")

    eventSet = dataSet(eventSetName, READ)
    tab = table(eventSet, 'EVENTS')
    call readWcs(tab, 'X', 'Y', wcs) ! in ssclib/wcs_aux.
    call release(tab)
    timeAttWcs%wcs = wcs

    call message(AppMsg, Verbose, "Miscellaneous setups.")

!    select case(dssStyle)
!    case('templateset','image')
!      dssSet = dataSet(dssSetName, READ)
!      dssBlock = block(dssSet, 0)
!      dssPointer = dataSubspace(dssBlock)
!    case('eventlist')
!      dssBlock = block(eventSet, 'EVENTS')
!      dssPointer = dataSubspace(dssBlock)
!    case('none')
!      dssPointer = dataSubspace()
!    case default
!call error('',"")
!    end select

    call getCalInfo(eventSet, instrumentId, expStartTimeStamp&
      , scAttitude, instrumAttitude) ! in ssclib/cal_aux.

!*********** compare eventSet and templateImage values of instrumentId, expStartTimeStamp, scAttitude??????

    timeAttWcs%timeStamp = expStartTimeStamp
    timeAttWcs%attitude  = scAttitude

    if (instrumentId .eq. EMOS1&
    .or. instrumentId .eq. EMOS2) then
      chipMinRawX = EMOS_MIN_X
      chipMaxRawX = EMOS_MAX_X
      chipMinRawY = EMOS_MIN_Y
      chipMaxRawY = EMOS_MAX_Y
      numCcds = 7
      numNodes = 2
      numCanonicalPatterns = 13
    else ! EPN.
      chipMinRawX = EPN_MIN_X
      chipMaxRawX = EPN_MAX_X
      chipMinRawY = EPN_MIN_Y
      chipMaxRawY = EPN_MAX_Y
      numCcds = 12
      numNodes = 1
      numCanonicalPatterns = 5
    end if

    allocate(ccd2ccdnr(numCcds, numNodes))
    allocate(ccdStr(numCcds, numNodes))

    maxCcdNodeNum = 10 * (numNodes - 1) + numCcds
    numCcdNodeCombos = numNodes * numCcds
    allocate(keyInfo(numCcdNodeCombos))

    do ccdNum = 1, numCcds
      do nodeNum = 1, numNodes
        ccdnr = 10 * (nodeNum - 1) + ccdNum
        ccdNodeCombo = nodeNum + numNodes * (ccdNum - 1)
        ccd2ccdnr(ccdNum, nodeNum) = ccdnr
        write(ccdStr(ccdNum, nodeNum), '(i2.2)') ccdnr
      end do
    end do

    if (outputStyle .eq. 'sky') then
      !.................................................................
      ! Construct the list of attitudes:

      call message(AppMsg, verbose, "Constructing attitude samples.")
      select case(attitudeStyle)
      case('binnedset')
        call splitSetTabName(binnedAttSetTabName, binnedAttSetName, binnedAttTabName)
        ! in ssclib/dal_aux.
        call readBinnedAttitude(binnedAttSetName, binnedAttTabName, binnedScAtt, noWeightColumn)
        ! in attcalc2/binned_att.
        numAttBins = size(binnedScAtt)

      case('template')
        numAttBins = 1
        allocate(binnedScAtt(1))
        call initializeBinnedAtt(binnedScAtt)
        binnedScAtt(1)%isGood = .true.
        binnedScAtt(1)%gtiDuration = 1.0d0 ! dummy value.
        binnedScAtt(1)%att = medianScAttitude

      case default
call error('badAttitudeStyle',"")
      end select

      allocate(binnedAttWeights(numCcdNodeCombos, numAttBins))
    end if

    !....................................................................
    ! Get info from event list:

    allocate(expInfo(numCcds, numNodes))
    call readCcdExposureInfo(eventSet, ccdStr, instrumentId&
      , expInfo, filtId) ! in exposure_map

    call CAL_setState(filterId=filtId)

    !....................................................................
    ! Get energy/spectral/exposure info (similar bit of code to
    ! lccorr_corrector):

    call message(AppMsg, Verbose, "Setting up various functions of energy.")

!!!    if (!** with spectrum) then
!!!      call getEnergyBounds(dssPointer, dssBlock, expInfo(1,1)%temperature&
!!!        , expStartTimeStamp, EeVRange, spectrum)
!!!    else

!      EeVRange = getEnergyBounds(dssPointer)
      calEeVRange = getEnergyBounds() ! just returns the cal range
      call andRanges(userEeVRanges, calEeVRange, EeVRanges)
      EeVRange%lower = EeVRanges(1)%lower
      EeVRange%upper = EeVRanges(size(EeVRanges))%upper

!!!    end if

    call sampleEnergyRange(EeVRange, EeVsamples) ! in ssclib/exposure.

!!!    if (!** with spectrum) then
!*** interpolate user spectrum
!!!    else
      call constructModelSpectrum(EeVsamples, spectrum) ! in exposure_map.
!!!    end if

    call getFilterTransmission(EeVsamples, filterTrans) ! in exposure.f90
    call getEffectiveArea(EeVsamples, effArea) ! in ssclib/exposure.
    EeVFunctions = spectrum * effArea * filterTrans
    vigEfunctionAtSrc = 1.0

    !....................................................................

    if (withVigExpMapSet .or. withVigExpCubeSet) then
      allocate(thetaArcsecSamples(numThetaSamples))
      allocate(phiSamples(numPhiSamples))
      allocate(vigSamplesCube(numESamples, numThetaSamples, numPhiSamples))

      thetaArcsecSamples = (/(i/real(numThetaSamples),i=0,numThetaSamples-1)/)
      thetaArcsecSamples = thetaArcsecSamples * 900.0d0
      phiSamples = (/(i/real(numPhiSamples),i=0,numPhiSamples-1)/)
      phiSamples = phiSamples * TwoPi

      call constructVignettingCube(EeVsamples, thetaArcsecSamples&
        , phiSamples, vigSamplesCube) ! in ssclib/exposure.

      allocate(vigSamples2d(numThetaSamples, numPhiSamples))

      do ti = 1, numThetaSamples
        do pi = 1, numPhiSamples
          vigSamples2d(ti, pi) = integrateRanges(real(EeVSamples, single)&
!            , vigSamplesCube(:,ti,pi), EeVRange)
            , vigSamplesCube(:,ti,pi), EeVRanges)
        end do
      end do

      deallocate(vigSamplesCube)

      vigSamples2d = vigSamples2d / vigSamples2d(1, 1)

      allocate(ccdXRawVec(chipMinRawX:chipMaxRawX))
      allocate(ccdYRawVec(chipMinRawY:chipMaxRawY))
      allocate(ccdXRaw(chipMinRawX:chipMaxRawX&
        , chipMinRawY:chipMaxRawY))
      allocate(ccdYRaw(chipMinRawX:chipMaxRawX&
        , chipMinRawY:chipMaxRawY))
      allocate(ccdPhi(chipMinRawX:chipMaxRawX&
        , chipMinRawY:chipMaxRawY))
      allocate(ccdThetaArcsec(chipMinRawX:chipMaxRawX&
        , chipMinRawY:chipMaxRawY))
      allocate(ccdVigMaps(numCcdNodeCombos, chipMinRawX:chipMaxRawX&
        , chipMinRawY:chipMaxRawY))

      ccdXRawVec = (/(i, i=chipMinRawX,chipMaxRawX)/)
      ccdXRaw = spread(ccdXRawVec, 2, 1+chipMaxRawY-chipMinRawY)
      ccdYRawVec = (/(i, i=chipMinRawY,chipMaxRawY)/)
      ccdYRaw = spread(ccdYRawVec, 1, 1+chipMaxRawX-chipMinRawX)
    end if ! with vignetting

    !....................................................................

    allocate(tempCcdExpMap(chipMinRawX:chipMaxRawX&
      , chipMinRawY:chipMaxRawY))
    allocate(ccdExpMaps(numCcdNodeCombos, chipMinRawX:chipMaxRawX&
      , chipMinRawY:chipMaxRawY))

    allocate(gtiVec(numCcdNodeCombos))
    allocate(qeArray(numCanonicalPatterns, numESamples))

    requiredCorrections = 0 ! default
    requiredCorrections = ibset(requiredCorrections, TREAT_GTI_BIT)
    requiredCorrections = ibset(requiredCorrections, TREAT_COSMIC_BIT)
    requiredCorrections = ibset(requiredCorrections, TREAT_DEAD_TIME_BIT)

    numImages = 0

    call message(AppMsg, Verbose, "vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv")
    call message(AppMsg, Verbose, "Main loop:")

    do ccdNum = 1, numCcds
      do nodeNum = 1, numNodes
      if (.not. expInfo(ccdNum, nodeNum)%exposuExists) cycle

        if (instrumentId == EPN) then ! just to stop the cal giving a warning:
          call CAL_setState(ccdChipId=int(ccdNum, int32))
        else
          call CAL_setState(ccdChipId=int(ccdNum, int32)&
            , ccdNodeId=int(nodeNum - 1, int32))
        end if

        ccdNodeNum = ccd2ccdnr(ccdNum, nodeNum)
        ccdNodeCombo = nodeNum + numNodes * (ccdNum - 1)

        call message(AppMsg, Verbose, " ********************************")
        call message(AppMsg, Verbose, " Found EXPOSUnn extension for &
          &CCD/node number nn = "//ccdStr(ccdNum, nodeNum))

        qePixelXi = nint((expInfo(ccdNum, nodeNum)%winMinRawX&
                        + expInfo(ccdNum, nodeNum)%winMaxRawX) / 2.0)
        qePixelYi = nint((expInfo(ccdNum, nodeNum)%winMinRawY&
                        + expInfo(ccdNum, nodeNum)%winMaxRawY) / 2.0)

        call constructQeArray(EeVsamples, qeArray, qePixelXi, qePixelYi)
        ! in ssclib/exposure.f90

        eNorm = calcEeVFunctionNorm(EeVsamples, spectrum * effArea&
          , filterTrans, vigEfunctionAtSrc, qeArray, requiredCorrections)

!        call calcCcdExpMap(eventSet, ccdNodeNum, dssPointer&
!          , timeAttWcs, pleaseCorrectDither, numMicropixelDivs&
!          , expInfo(ccdNum, nodeNum), EeVsamples, EeVfunctions, qeArray&
!          , ccdExpMap=tempCcdExpMap, ccdFlatExp=ccdFlatExp)
        call calcCcdExpMap(tempEventListSetName, eventSet, ccdNodeNum&
          , rawYLo, patternLo, patternHi, flagMask, EeVRanges&
          , timeAttWcs, pleaseCorrectDither, numMicropixelDivs&
          , expInfo(ccdNum, nodeNum), EeVsamples, EeVfunctions, qeArray&
          , ccdExpMap=tempCcdExpMap, ccdFlatExp=ccdFlatExp)
        ! in exposure_map_pcms.

      if (all(tempCcdExpMap == 0.0)) cycle

        numImages = numImages + 1
        keyInfo(numImages)%ccdNum = ccdNum
        keyInfo(numImages)%nodeNum = nodeNum
!        ccdNumList(numImages) = ccdNum

        ccdExpMaps(numImages,:,:) = tempCcdExpMap&
          * expInfo(ccdNum, nodeNum)%intFraction

!        if (hasTimeFilter(dssPointer, ccdNodeNum)) then
!          call readGtisFromDss(dssSet, dssPointer, int(ccdNodeNum)&
!            , gtiVec(numImages)%gtis) ! in ssclib/dss_aux.
        if (withGtis) then
          call readGtis(userGtiSetTabNames(ccdNum, nodeNum)&
            , gtiVec(numImages)%gtis) ! in ssclib/intervals_aux.

          keyInfo(numImages)%gtiTableName&
            = 'GTI000'//ccdStr(ccdNum, nodeNum)

        else
!******* warn. This will give problems if numAttBins > 1.
          allocate(gtiVec(numImages)%gtis(0))
          keyInfo(numImages)%gtiTableName = ''
        end if

        call gtiWeightBinnedAttitude(gtiVec(numImages)%gtis, binnedScAtt)
        ! in attcalc2/binned_att.

        binnedAttWeights(numImages,:) = binnedScAtt%gtiDuration
!********** normalize?????

        ccdExpMaps(numImages,:,:) = ccdExpMaps(numImages,:,:) / eNorm

        if (withVigExpMapSet .or. withVigExpCubeSet) then
          !..............................................................
          ! Get chip map of vignetting function:

          call rawToTel(ccdXRaw, ccdYRaw, ccdThetaArcsec, ccdPhi)
          ! in ssclib/coordinate.

          call interpolateVigMapToArray(ccdPhi, ccdThetaArcsec&
            , thetaArcsecSamples, phiSamples, vigSamples2d&
            , ccdVigMaps(numImages,:,:))

          ccdVigMaps(numImages,:,:) = ccdVigMaps(numImages,:,:)&
            * ccdExpMaps(numImages,:,:)
        end if ! with vignetting
      end do ! nodeNum
    end do ! ccdNum

if (numImages <= 0) call error('allMapsZero',"")

!    if (dssStyle=='templateset' .or. dssStyle=='image')&
!      call release(dssSet) ! need it until now for the dss.

    call release(eventSet)
!    call dssRelease(dssPointer)

    !....................................................................

    call message(AppMsg, Verbose, "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^")
    call message(AppMsg, Verbose, "Now saving the exposure map.")

    select case(outputStyle)
    case('sky')
!      call writeCubeToSkyImage(ccdExpMaps(:numImages,:,:)&
!        , keyInfo(:numImages), expStartTimeStamp&
!!        , gtiVec(:numImages), templateImageSetName, dssSetName&
!, gtiVec(:numImages), templateImageSetName, templateImageSetName&
!        , flatExpMapSetName&
!!        , tempImageSetName, 'rate', .true., wcs, doApprox, attitudeStyle&
!!        , binnedAttSetName)
!        , tempImageSetName, 'rate', wcs, doApprox, attitudeStyle&
!        , binnedAttSetName, binnedAttTabName)
!      ! in eimchip2sky/eimchip2sky_aux.

      call rebinChipImagesToSky(ccdExpMaps(:numImages,:,:), keyInfo(:numImages)%ccdNum&
        , wcs, binnedScAtt%att, binnedAttWeights(:numImages,:), expStartTimeStamp&
        , 'rate', doApprox, flatExpMap)

      call saveImage(flatExpMap, templateImageSetName, flatExpMapSetName&
        , tempImageSetName, wcs=wcs) ! in ssclib/save_image.

      if (withVigExpMapSet) then
!        call writeCubeToSkyImage(ccdVigMaps(:numImages,:,:)&
!          , keyInfo(:numImages), expStartTimeStamp&
!!          , gtiVec(:numImages), templateImageSetName, dssSetName&
!, gtiVec(:numImages), templateImageSetName, templateImageSetName&
!          , vigExpMapSetName&
!!          , tempImageSetName, 'rate', .true., wcs, doApprox, attitudeStyle&
!!          , binnedAttSetName)
!          , tempImageSetName, 'rate', wcs, doApprox, attitudeStyle&
!          , binnedAttSetName, binnedAttTabName)
!        ! in eimchip2sky/eimchip2sky_aux.

        call rebinChipImagesToSky(ccdVigMaps(:numImages,:,:)&
          , keyInfo(:numImages)%ccdNum, wcs, binnedScAtt%att&
          , binnedAttWeights(:numImages,:), expStartTimeStamp, 'rate', doApprox&
          , vigExpMap)

        call saveImage(vigExpMap, templateImageSetName, vigExpMapSetName&
          , tempImageSetName, wcs=wcs) ! in ssclib/save_image.
      end if

      deallocate(binnedScAtt)

    case('raw')
      call writeCubeData(flatExpMapCubeSetName, templateImageSetName&
        , ccdExpMaps(:numImages,:,:), keyInfo(:numImages), gtiVec(:numImages))

      if (withVigExpCubeSet) then
        call writeCubeData(vigExpMapCubeSetName, templateImageSetName&
          , ccdVigMaps(:numImages,:,:), keyInfo(:numImages), gtiVec(:numImages))
      end if
    case default
call error('',"")
    end select

    ! Deallocate all the gtis:

    do ccdNum = 1, numCcds
      do nodeNum = 1, numNodes
        ccdNodeCombo = nodeNum + numNodes * (ccdNum - 1)
      if (.not. associated(gtiVec(ccdNodeCombo)%gtis)) cycle
        deallocate(gtiVec(ccdNodeCombo)%gtis)
      end do
    end do
!****** can rather use deallocateGtiVec in ssclib/intervals_aux?

    deallocate(EeVRanges)

  end subroutine eexpchipmap_pcms

  !---------------------------------------------------------------------
  subroutine readParameters
  !---------------------------------------------------------------------

    implicit none 

    integer :: i, gtiNum

    !-------------------------------------------------------------------
    ! Start:

    eventSetName = stringParameter('eventset')
    templateImageSetName = stringParameter('templateset')
    tempEventListSetName = stringParameter('tempeventset')
!!!    withSpectrumSet = booleanParameter('withspectrumset')
!!!    if (withSpectrumSet)&
!!!      spectrumSetTabName = stringParameter('spectrumset')

    outputStyle = stringParameter('outputstyle')
    select case(outputStyle)
    case('sky')
      flatExpMapSetName = stringParameter('flatexpimgset')
      withVigExpMapSet = booleanParameter('withvigexpimgset')
      if (withVigExpMapSet)&
        vigExpMapSetName = stringParameter('vigexpimgset')

      attitudeStyle = stringParameter('attstyle')
      select case(attitudeStyle)
      case('binnedset')
        binnedAttSetTabName = stringParameter('binnedatttab')

      case('template')
      case default
call error('',"")
      end select

    case('raw')
      flatExpMapCubeSetName = stringParameter('flatexpcubeset')
      withVigExpCubeSet = booleanParameter('withvigexpcubeset')
      if (withVigExpCubeSet)&
        vigExpMapCubeSetName = stringParameter('vigexpcubeset')
    case default
call error('',"")
    end select

!    withDss = booleanParameter('withdss')
!    if (withDss) then
!      dssStyle = stringParameter('dssstyle')
!      select case(dssStyle)
!      case('templateset')
!!if (outputStyle .ne. 'sky') call error('notAvailable',"")
!        dssSetName = templateImageSetName
!      case('image')
!        dssSetName = stringParameter('dssset')
!      case('eventlist')
!      case default
!call error('',"")
!      end select
!    else
!      dssStyle = 'none'
!    end if

!    tempImageSetName = stringParameter('tempset')
tempImageSetName = 'temp.ds'
!    doApprox = booleanParameter('usefast')
doApprox = .true.

patternLo = intParameter('patternlo')
patternHi = intParameter('patternhi')
flagMaskHexStr = stringParameter('flagmask')
numUserEeVRanges = parameterCount('pilos')
if (numUserEeVRanges <= 0) call error('',"")
if (numUserEeVRanges .ne. parameterCount('pihis')) call error('',"")
allocate(userEeVRanges(numUserEeVRanges))
userEeVRanges%lower%type = INCLUSIVE
userEeVRanges%upper%type = INCLUSIVE
do i = 1, numUserEeVRanges
  userEeVRanges(i)%lower%value = intParameter('pilos', i-1)
  userEeVRanges(i)%upper%value = intParameter('pihis', i-1)
end do

withGtis = booleanParameter('withgtis')
if (withGtis) then
        numGtiTabs = parameterCount('gtitabsnode0')
        withNode1 = booleanParameter('withgtitabsnode1')
        if (withNode1) then
          if (parameterCount('gtitabsnode1') .ne. numGtiTabs)&
call error('wrongNumGtiTabsNode1',"")
          allocate(userGtiSetTabNames(numGtiTabs, 2))
        else
          allocate(userGtiSetTabNames(numGtiTabs, 1))
        end if

        do gtiNum = 1, numGtiTabs
          userGtiSetTabNames(gtiNum, 1) = stringParameter('gtitabsnode0'&
            , gtiNum-1)
        end do

        if (withNode1) then
          do gtiNum = 1, numGtiTabs
            userGtiSetTabNames(gtiNum, 2) = stringParameter('gtitabsnode1'&
              , gtiNum-1)
          end do
        end if
end if
rawYLo = intParameter('rawylo')

    applyAttcalcFudge = booleanParameter('withboresightfudge')
    ! only needed while boresight psi has wrong sign. IMS 2005-11-17

  end subroutine readParameters

  !---------------------------------------------------------------------
  subroutine template
  !---------------------------------------------------------------------

    implicit none 

    !-------------------------------------------------------------------
    ! Start:

  end subroutine template

end module eexpchipmap_pcms_mod

